@rem = '--*-Perl-*--
@echo off
if "%OS%" == "Windows_NT" goto WinNT
perl -x -S "%0" %1 %2 %3 %4 %5 %6 %7 %8 %9
goto endofperl
:WinNT
perl -x -S "%0" %*
if NOT "%COMSPEC%" == "%SystemRoot%\system32\cmd.exe" goto endofperl
if %errorlevel% == 9009 echo You do not have Perl in your PATH.
goto endofperl
@rem ';
#!/usr/local/bin/perl5
#line 14
#
#	ms2pred.pl	Microsoft Developer Studio Project file to PrEditor Project
#				file converter
#
#	Written by John Young, 14 February 2002
#	
# DESCRIPTION
#	
#	This script takes as input a Microsoft format project file, either a 
#	workspace (.dsw) file, or a project file (.dsp).  It then parses it and 
#	produces an equivalent PrEditor project file (.prj).  
#
#	The format of a Microsoft project file is as follows...
#
#
#
# HISTORY
#	Ver	Date		By	Description
#	---------------------------------------------------------------------------
#	1.0	14-Feb-02	JY	Created
#	1.1  30-Apr-02	JY	Support for configurations added
#	1.2	12-Feb-03 JY	Fixed bug for top level items, and changed environment 
#					variables so that the project name is not included.


use strict;

(my $scriptName = $0) =~ s/(.*\\)?(.*)\.pl/$2/;       	# Remove the .pl
my $version = "1.1";

sub usage
{
     print $_[0] if defined ($_[0]);
"\
\
usage:  \
     perl $scriptName.pl [-hvVx] [-o <pred-proj>] <ms-proj>\
\
     where\
          <ms-proj>           The Microsoft project name.  This can be either a\
                              workspace file (.dsw), or a project file (.dsp).\
                              A workspace file generally references one or more\
                              .dsp files.\
\
          -o <pred-proj>      The name of the PrEditor project file to use.  If\
                              the extension is omitted, then \".prj\" is \
                              appended.  If this -o option is not present, then\
                              the base filename of the <ms-proj> is used with a\
                              .prj extension.\
\
          -x                  Overwrite files without asking.  Normally, \
                              ms2pred will not overwrite an existing file.  \
                              This switch will cause ms2pred to overwrite \
                              files, so please use with caution.\
\
          -h                  Displays the usage (this message)\
          -v                  Verbose\
          -V                  Display version number\n";

}


# Deal with the command line...
use Getopt::Std;
use vars qw ($opt_h $opt_o $opt_v $opt_V $opt_x);
getopts ('ho:vVx');
print &usage and exit (0) if $opt_h;

if ( $opt_V )
{
     print "\
     $scriptName $version\
     MS project to PrEditor project convertor $version\
     Copyright (c) 2002 Young Associates\
     All rights reserved\n";
     exit (0);
}

my $verbose = $opt_v;
my $prjFile = $opt_o;
my $overwrite = $opt_x;
my $msFile = shift;

die usage ("No project file specified") if $msFile eq "";

if ( !$prjFile )
{
	my $i = rindex ($msFile, ".");
	$prjFile = $i == -1 ? $msFile : substr ($msFile, 0, $i);
	$prjFile .= ".prj";	
}

print "Converting project '$msFile' to '$prjFile\n" if $verbose;

my $i = rindex ($msFile, ".");
my $ext = substr ($msFile, $i) if $i != -1;

my $active;

if ( $ext eq ".dsw" )
{
	if ( $prjFile eq "" )
	{
		$prjFile = GetName ($msFile) . "-master.prj";
	}
	ProcessDSW ($msFile, $prjFile);
}
else
{
	if ( $prjFile eq "" )
	{
		$prjFile = GetName ($msFile) . ".prj";
	}
	my $project = GetProject ($msFile);
	if ( CheckFile ($prjFile) )
	{
		CreateProject ($project, $prjFile);
	}
	else
	{
		print "$prjFile already exists and I will not overwrite it.\n";
		print "Use the -x switch to overwrite the file\n";
	}
}

print "Done!" if $verbose;

exit (0);



#	ProcessDSW ($dswFile, $prjFile);
#
#	$dwsFile is the name of the DSW file to create a PrEditor project from.
#	If the output file is omitted, then the output file is taken from the given
#	DSP file and given a '.prj' extension.
sub ProcessDSW
{
	my ($dswFile, $prjFile) = @_;

	if ( $dswFile eq "" )
	{
		die "Cannot create a project from an empty file name!";
	}

	# Get the project name...
	if ( $prjFile eq "" )
	{
		# Form up the workspace file name from the given $dswFile.
		my $workspace = GetName ($dswFile);
		$prjFile = $workspace . ".prj";	
	}
	
	print "Converting '$dswFile' to '$prjFile'\n" if $verbose;

	my %projects;
	open DSW, "<$dswFile" or die "Couldn't open file '$dswFile'\n";
	while ( <DSW> )
	{
		chomp;
		next if /^\s*$/;		# Skip blank lines
		
		if ( /^Project: "(.*?)"=(.*?) - Package Owner=/ )
		{
			my $project = $1;
			my $path = $2;
			$projects {$project} = $path;
		}
	}
	
	if ( $verbose )
	{
		print "Projects in workspace '$dswFile'\n";
		for my $project (sort keys %projects)
		{
			print "   '$project'      '$projects{$project}'\n";
		}
	}

	if ( !$overwrite )
	{
		my @existingFiles;
		push @existingFiles if -e $prjFile;
		foreach my $project (sort keys %projects)
		{
			my $prjFile = $projects {$project};
			my $i = rindex ($prjFile, ".");
			$prjFile = $i == -1 ? $prjFile : substr ($prjFile, 0, $i);
			$prjFile .= ".prj";
			print "Checking file '$prjFile'\n";
			if ( -e $prjFile )
			{
				push @existingFiles, $prjFile;
			}
		}
		
		if ( @existingFiles )
		{
			print "The files...\n";
			foreach my $file (@existingFiles)
			{
				print "   $file\n";
			}
			print " already exist and I will not overwrite them.\n";
			print "Use the -x switch.\n";
			return 0;
		}
	}
	

	# Now create the 'master' project...
	open (PRJ, ">$prjFile") or die "Couldn't create PrEditor project file '$prjFile'\n";
	
	my $projname = GetName ($prjFile);
	print PRJ "<?xml version=\"1.0\"?>\n";
	print PRJ "<Project version=\"2\" magic-number=\"0xaf41b307\">\n";
	print PRJ "<ToolsCmds>\n";
	print PRJ "</ToolsCmds>\n";
	print PRJ "<ProjectTree>\n";
	print PRJ "<Branch name=\"$projname\" type=\"Title\" expanded=\"yes\">\n";

	
	foreach my $project (sort keys %projects)
	{
		my $dspFile = $projects {$project};
		my $prjFile = GetName ($dspFile) . ".prj";
		print PRJ "<Branch name=\"$project\" type=\"Title\" path=\"$prjFile\" expanded=\"no\">\n";
		print PRJ "</Branch>\n";
	}

	print PRJ "</Branch>\n";
	print PRJ "</ProjectTree>\n";
	print PRJ "<Workspace>\n";
	print PRJ "</Workspace>\n";
	print PRJ "</Project>\n";

	# Now the sub-projects...
	foreach my $project (sort keys %projects)
	{
		my $dspFile = $projects {$project};
		my $prjFile = GetName ($dspFile) . ".prj";
		my $rProject = GetProject ($dspFile);
		CreateProject ($rProject, $prjFile);
	}
}





#	$rGroups = GetProject ($dspFile);
#
#	$dspFile is the name of the DSP file to create a PrEditor project from.
#	If the output file is omitted, then the output file is taken from the given
#	DSP file and given a '.prj' extension.
#
#	\return	A reference to a hash of project names against project groups.  
#			Each group is a simple list of path names which are usually the 
#			source files that make up the project.
#
#	$rGroup looks like this...
#
#		Key				Value
#		--------------------------------------------------------------
#		Project name		List of path names of files in the project
#
sub GetProject
{
	my ($dspFile) = @_;

	if ( $dspFile eq "" )
	{
		die "Cannot create a project from an empty file name!";
	}


	my $project = GetName ($dspFile);
	print "Reading project '$project' (file '$dspFile')\n" if $verbose;

     # 'Constants'
     my $PROJECT = "Project";
     my $TARGET  = "Target";
     my $GROUP   = "Group";
     my $SOURCE  = "Source File";

     print "Opening '$dspFile'\n" if $verbose;


	my %project =  (
				 CONFIGS => [ ],	# A list of configurations
				 GROUPS  => { }	# A table of groups with source files.
				);


	my $configs = $project{"CONFIGS"};
	my $groups  = $project{"GROUPS"};
	
	

     my $section;
     my $group;

	open (DSP, "<$dspFile") or die "Couldn't open file '$dspFile'\n";
	while ( <DSP> )
	{
		chomp;
		next if /^\s*$/;		# Skip blank lines

#          print "$_\n";

		my $line = $_;
          if ( $line =~ /^# Begin (.*)/ )
          {
               my $s = $1;
               if ( $s =~ /(Group) "(.*)"/ )
               {
                    $section = $1;
	               $group = $2;
	               print "Parsing $section '$group'\n" if $verbose;
               }
               else
               {
                    $section = $s;
#                    $group = "";
	               print "Parsing $section\n" if $verbose;
               }
          }
          elsif ( $line =~ /^# End (.*)/ )
          {
			my $end = $1;
               if ( $end eq $SOURCE )
               {
                    $section = $GROUP;
               }
               elsif ( $end eq $GROUP )
               {
                    $section = $TARGET;
               }
               elsif ( $end eq $TARGET )
               {
                    $section = $PROJECT;
               }
               else
               {
                    $section = "";
               }
               print "End of $end\n" if $verbose;
          }
          else
          {
               if ( $section eq $SOURCE )
               {
               	my $srcFile;
               	$srcFile = $1 if $line =~ /^SOURCE=(.*)/;
				$srcFile = $1 if $srcFile =~ /"(.*)"/;		# get rid of quotes
               	if ( $srcFile ne "" )
               	{
					if ( !exists $groups->{$group} )
					{
						$groups->{$group} = [ ];	# create an emtpy list.
					}
					print "   Adding $srcFile to group '$group'\n" if $verbose;
	               	push @{$groups->{$group}}, $srcFile;
				}
               }
               elsif ( $section eq $GROUP )
               {
               }
               elsif ( $section eq $TARGET )
               {
                    if ( /# Name "(.*)"/ )
                    {
                         my $config = $1;
         
         				# Most likely something like "PrEditor - Win32 Debug".
         				# Delete the project name from it.
         				$config = $' if $config =~ /^$project - /i;
                         print "   Target '$config'\n" if $verbose;
                         push @{$configs}, $config;
                    }
               }
               elsif ( $section eq $PROJECT )
               {
#                    if ( /!(?:ELSE)?IF\s+"\$\(CFG\)" == "(.*)"/ )
#                    {
#                         $currCfg = $1;
#                         print "Processing configuration '$currCfg'\n" if $verbose;
#                    }
               }
          }
     }


	# Now create some output
	if ( $verbose )
	{
		print "\n\nProject \"$project\"\n";
		foreach my $config (@{$configs})
		{
			print "   Configuration \"$config\"\n";
		}
		
		foreach my $group (sort keys %{$groups})
		{
			print "Group \"$group\"\n";
			for my $file (sort @{$groups->{$group}})
			{
				print "   \"$file\"\n";
			}
		}
	}

	#return \%{$groups};
	return \%project;
}



#	CreateProject (\%project, $prjFile);
#
#	\%project is a reference to a project (see above for a description).
#	$prjFile is the file to create.
#
sub CreateProject
{
	my ($rProject, $prjFile) = @_;

	my $rGroups  = $rProject->{"GROUPS"};
	my $rConfigs = $rProject->{"CONFIGS"};

	if ( $prjFile eq "" )
	{
		die "Cannot create a project from an empty file name!";
	}

	# Get the project name...	
	my $i = rindex ($prjFile, ".");
	my $project = $i == -1 ? $prjFile : substr ($prjFile, 0, $i);


	print "Creating '$prjFile'\n" if $verbose;

	if ( %{$rGroups} == 0 )
	{
		print STDERR "No files found in project \"$project\"\n";
		return 0;
	}

	print "Writing '$prjFile\n" if $verbose;
	open (PRJ, ">$prjFile") or die "Couldn't create PrEditor project file '$prjFile'\n";
	
	print PRJ "<?xml version=\"1.0\"?>\n";
	print PRJ "<Project version=\"2\" magic-number=\"0xaf41b307\">\n";
	print PRJ "<ToolsCmds>\n";
	
	#Compile project tool
	print PRJ "<ToolCmd menu=\"&amp;Compile\" cmd=\"nmake\" ";
	print PRJ "args=\"/nologo /f &quot;%N.mak&quot; &quot;%n.obj&quot; CFG=&quot;%N - %C&quot;\" ";
	print PRJ "file-regex=\"\\s*(.*?)\\(\" line-regex=\"\\((\\d*)\" ";
	print PRJ "use-output-window=\"Yes\" prompt-for-args=\"No\" close-on-exit=\"No\"/>\n";
	
	#'Build' project tool
	print PRJ "<ToolCmd menu=\"&amp;Build\" cmd=\"nmake\" ";
	print PRJ "args=\"/nologo /f &quot;%N.mak&quot; CFG=&quot;%N - %C&quot;\" ";
	print PRJ "file-regex=\"\\s*(.*?)\\(\" line-regex=\"\\((\\d*)\" ";
	print PRJ "use-output-window=\"Yes\" prompt-for-args=\"No\" close-on-exit=\"No\" />\n";

	#'Rebuild' project tool
	print PRJ "<ToolCmd menu=\"&amp;Rebuild\" cmd=\"nmake\" ";
	print PRJ "args=\"/nologo /a /f &quot;%N.mak&quot; CFG=&quot;%N - %C&quot; clean all\" ";
	print PRJ "file-regex=\"\\s*(.*?)\\(\" line-regex=\"\\((\\d*)\" ";
	print PRJ "use-output-window=\"Yes\" prompt-for-args=\"No\" close-on-exit=\"No\" />\n";
	
	#'Clean' project tool
	print PRJ "<ToolCmd menu=\"C&amp;lean\" cmd=\"nmake\" ";
	print PRJ "args=\"/nologo /f &quot;%N.mak&quot; CFG=&quot;%N - %C&quot; clean\" ";
	print PRJ "file-regex=\"\" line-regex=\"\" ";
	print PRJ "use-output-window=\"Yes\" prompt-for-args=\"No\" close-on-exit=\"No\" />\n";

	print PRJ "</ToolsCmds>\n";

	# Configurations
	print PRJ "<Configurations>\n";
	for my $config (sort @{$rConfigs})
	{
		print PRJ "<Config name=\"$config\"/>\n";
	}
	print PRJ "</Configurations>\n";
	

	print PRJ "<ProjectTree>\n";
	print PRJ "<Branch name=\"$project\" type=\"Title\" expanded=\"yes\"";
	if ( $active eq "" )
	{
		$active = $project;
		print PRJ " active=\"yes\"";
	}
	print PRJ ">\n";

	
	foreach my $group (sort keys %{$rGroups})
	{
		print PRJ "<Branch name=\"$group\" type=\"Heading\" expanded=\"no\">\n"
				if $group;
		for my $file (sort @{$rGroups->{$group}})
		{
			#print "\$file='$file'\n";
			my $i = rindex ($file, "\\");
			my $name = $i == -1 ? $file : substr ($file, $i + 1);
			#print "\$name='$name'\n";
			print PRJ "<ProjectItem type=\"Relative Filename\" ";
			print PRJ "name=\"$name\" path=\"$file\"/>\n";
		}
		print PRJ "</Branch>\n"	if $group;

	}

	print PRJ "</Branch>\n";
	print PRJ "</ProjectTree>\n";
	print PRJ "<Workspace>\n";
	print PRJ "</Workspace>\n";
	print PRJ "</Project>\n";
	return 1;
}


#	$ret = CheckFile ($file)
#
#	$file is the name of the file to check.  If the file exists, then prompt
#	the user to overwrite - if the $overwrite flag is not set.
sub CheckFile
{
	my ($file) = @_;

	return 1 if $overwrite;
	return 1 if !-e $file;
	return 0;
}



#	$name = GetName ($path);
#
#	Returns the name of the given file (i.e. minus the extension).
#
sub GetName
{
	my ($path) = @_;

	my $i = rindex ($path, "\\");
	my $fileName = $i == -1 ? $path : substr ($path, $i);
	$i = rindex ($path, "."); 
	my $name = $i == -1 ? $path : substr ($path, 0, $i);
	return $name;
}



__END__
__END__
:endofperl
